==> MT-1.1.tar.gz: MT-1.1/lib/MT/App.pm <==
# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: App.pm,v 1.46 2001/10/18 16:58:30 btrott Exp $

package MT::App;
use strict;

use File::Spec;
use HTML::Template;
use POSIX qw( strftime );
use CGI;
use CGI::Cookie;

use MT::Log;
use MT::Util qw( encode_html offset_time_list );
use MT;
use base qw( MT );

sub init {
    my $app = shift;
    $app->SUPER::init(@_);
    $app->{vtbl} = { };
    $app->{requires_login} = 0;
    $app->{query} = CGI->new;
    $app->{template_dir} = '';
    $app->{headers} = { };
    $app->{cookies} = CGI::Cookie->fetch;
    $app;
}

sub is_authorized { 1 }

sub login {
    my $app = shift;
    my $q = $app->{query};
    my $cookies = $app->{cookies};
    my($user, $pass, $remember, $crypted);
    my $first_time = 0;
    if ($cookies->{user}) {
        ($user, $pass, $remember) = split /::/, $cookies->{user}->value;
        $crypted = 1;
    } else {
        $first_time = 1;
        $user = $q->param('username');
        $pass = $q->param('password');
    }
    return unless $user && $pass;
    require MT::Author;
    if (my $author = MT::Author->load({ name => $user })) {
        if ($author->is_valid_password($pass, $crypted)) {
            if ($first_time) {
                $app->log("User '" . $author->name . "' logged in successfully");
            }
            return($author, $first_time);
        }
    }
    ## Login invalid, so get rid of cookie (if it exists) and let the
    ## user know.
    $app->log("Invalid login attempt from user '$user'");
    $app->bake_cookie(-name => 'user', -value => '', -expires => '-1y')
        unless $first_time;
    return $app->error("Invalid login.");
}

sub set_header {
    my $app = shift;
    my($key, $val) = @_;
    $app->{headers}{$key} = $val;
}

sub bake_cookie {
    my $app = shift;
    my %param = @_;
    unless ($param{-path}) {
        $param{-path} = $app->path;
    }
    my $cookie = CGI::Cookie->new(%param);
    $app->set_header('-cookie', $cookie);
}

sub show_error {
    my $app = shift;
    my($error) = @_;
    my $tmpl;
    $error = encode_html($error);
    $error =~ s!(http://\S+)!<a href="$1" target="_blank">$1</a>!g;
    $tmpl = $app->load_tmpl('error.tmpl') or
        return "Can't load error template; got error '" . $app->errstr .
               "'. Giving up. Original error was <pre>$error</pre>";
    $tmpl->param(ERROR => $error);
    $tmpl->output;
}

sub run {
    my $app = shift;
    my $q = $app->{query};
    my($body);
    eval {
    REQUEST:
    {
        if ($app->{requires_login}) {
        LOGIN:
        {
            my($author, $first_time) = $app->login;
            if ($author) {
                $app->{author} = $author;
                if ($first_time) {
                    my $remember = $q->param('remember') ? 1 : 0;
                    my %arg = (
                        -name => 'user',
                        -value => join('::', $author->name, $author->password,
                                             $remember),
                    );
                    $arg{-expires} = '+10y' if $remember;
                    $app->bake_cookie(%arg);
                }
                last LOGIN if $app->is_authorized;
            }
            my $tmpl = $app->load_tmpl('login.tmpl') or
                $body = $app->show_error( $app->errstr ), last REQUEST;
            $tmpl->param(ERROR => $app->errstr);
            ## For bookmarklets, we need to pass-thru text, title, and
            ## href vars
            if ($q->param('is_bm')) {
                $tmpl->param(text => $q->param('text'));
                $tmpl->param(link_title => $q->param('link_title'));
                $tmpl->param(link_href => $q->param('link_href'));
                $tmpl->param(bm_show => $q->param('bm_show'));
            }
            $body = $tmpl->output;
            last REQUEST;
        }  ## end LOGIN block
        }

        my $mode = $q->param('__mode') || $app->{default_mode};
        my $code = $app->{vtbl}{$mode} or
            $app->error("Unknown action $mode");
        if ($code) {
            $body = $code->($app);
        }
        unless (defined $body || $app->{redirect}) {
            $body = $app->show_error( $app->errstr );
        }
    }  ## end REQUEST block
    };
    if ($@) {
        $body = $app->show_error($@);
    }

    ## Add the Pragma: no-cache header.
    ## WEIRD: for CGI::Cache, any true argument to cache means NO cache
    $q->cache(1);

    if (my $url = $app->{redirect}) {
        print $q->redirect(-uri => $url, %{ $app->{headers} });
    } else {
        print $q->header(%{ $app->{headers} });
        $body = "<pre>$app->{trace}</pre>" . $body if $app->{trace};
        print $body;
    }
}

sub load_tmpl {
    my $app = shift;
    my($file, @p) = @_;
    my $path = $app->{cfg}->TemplatePath;
    my $tmpl;
    eval {
        $tmpl = HTML::Template->new_file(
            File::Spec->catfile($path, $app->{template_dir}, $file),
            path => [ File::Spec->catdir($path, $app->{template_dir}) ],
            die_on_bad_params => 0, global_vars => 1, @p);
    };
    return $app->error("Loading template '$file' failed: $@") if $@;
    my $blog_id = $app->{query}->param('blog_id');
    $tmpl->param(SCRIPT_URL => $ENV{SCRIPT_NAME});
    $tmpl->param(SCRIPT_PATH => $app->path);
    $tmpl->param(SCRIPT_FULL_URL => $app->{query}->url);
    if ($app->{author}) {
        $tmpl->param(AUTHOR_ID => $app->{author}->id);
        $tmpl->param(AUTHOR_NAME => $app->{author}->name);
    }
    if ($blog_id) {
        my $blog = MT::Blog->load($blog_id);
        $tmpl->param(BLOG_NAME => $blog->name);
        $tmpl->param(BLOG_ID => $blog_id);
        $tmpl->param(BLOG_URL => $blog->site_url);
    }
    if ($app->{query}->param('is_bm')) {
        $tmpl->param(is_bookmarklet => 1);
    }
    my $spath = $app->{cfg}->StaticWebPath || $app->path;
    $tmpl->param(static_uri => $spath);
    my $vers = MT->VERSION;
    if ($vers =~ /^0\.0*/) {
        $vers = "b $vers";
    }
    $tmpl->param(mt_version => $vers);
    $tmpl->param(DATE_NOW_FORMATTED =>
        strftime("%Y.%m.%d", offset_time_list(time, $blog_id)));
    $tmpl;
}

sub build_page {
    my $app = shift;
    my($file, $param) = @_;
    my $tmpl = $app->load_tmpl($file) or return;
    for my $key (keys %$param) {
        $tmpl->param($key, $param->{$key});
    }
    $tmpl->output;
}

sub uri { $ENV{SCRIPT_NAME} }
sub path {
    my $app = shift;
    unless ($app->{__path}) {
        ($app->{__path} = $app->uri) =~ s![^/]*$!!;
    }
    $app->{__path};
}

sub redirect {
    my $app = shift;
    my($url) = @_;
    unless ($url =~ m!^http://!i) {
        $url = $app->{query}->url(-base => 1) . $url;
    }
    $app->{redirect} = $url;
    return;
}

sub log {
    my $app = shift;
    my($msg) = @_;
    my $log = MT::Log->new;
    $log->message($msg);
    $log->ip($ENV{REMOTE_ADDR});
    $log->save;
}

sub trace { $_[0]->{trace} .= "@_" }

1;

==> MT-1.4.tar.gz: MT-1.4/lib/MT/App.pm <==
# Copyright 2001, 2002 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: App.pm,v 1.54 2002/01/05 22:46:07 btrott Exp $

package MT::App;
use strict;

use File::Spec;
use HTML::Template;
use POSIX qw( strftime );
use CGI;

use MT::Log;
use MT::Util qw( encode_html offset_time_list );
use MT;
@MT::App::ISA = qw( MT );

sub handler ($$) {
    my $class = shift;
    my($r) = @_;
    require Apache::Constants;
    my $app = $class->new( Config => $r->dir_config('MTConfig') );
    $app->{apache} = $r;
    my $html = $app->run(1);
    $r->send_http_header('text/html');
    $r->print($html);
    return Apache::Constants::OK();
}

sub init {
    my $app = shift;
    $app->SUPER::init(@_);
    $app->{vtbl} = { };
    $app->{requires_login} = 0;
    $CGI::POST_MAX = $app->{cfg}->CGIMaxUpload;
    $app->{query} = CGI->new;
    $app->{template_dir} = '';
    $app->{headers} = { };
    $app->{cookies} = $app->cookies;
    $app;
}

sub is_authorized { 1 }

sub login {
    my $app = shift;
    my $q = $app->{query};
    my $cookies = $app->{cookies};
    my($user, $pass, $remember, $crypted);
    my $first_time = 0;
    if ($cookies->{user}) {
        ($user, $pass, $remember) = split /::/, $cookies->{user}->value;
        $crypted = 1;
    } else {
        $first_time = 1;
        $user = $q->param('username');
        $pass = $q->param('password');
    }
    return unless $user && $pass;
    require MT::Author;
    if (my $author = MT::Author->load({ name => $user })) {
        if ($author->is_valid_password($pass, $crypted)) {
            if ($first_time) {
                $app->log("User '" . $author->name . "' logged in successfully");
            }
            return($author, $first_time);
        }
    }
    ## Login invalid, so get rid of cookie (if it exists) and let the
    ## user know.
    $app->log("Invalid login attempt from user '$user'");
    $app->bake_cookie(-name => 'user', -value => '', -expires => '-1y')
        unless $first_time;
    return $app->error("Invalid login.");
}

sub set_header {
    my $app = shift;
    my($key, $val) = @_;
    $app->{headers}{$key} = $val;
}

sub bake_cookie {
    my $app = shift;
    my %param = @_;
    unless ($param{-path}) {
        $param{-path} = $app->path;
    }
    if ($ENV{MOD_PERL}) {
        require Apache::Cookie;
        my $cookie = Apache::Cookie->new($app->{apache}, %param);
        $cookie->bake;
    } else {
        require CGI::Cookie;
        my $cookie = CGI::Cookie->new(%param);
        $app->set_header('-cookie', $cookie);
    }
}

sub cookies {
    my $app = shift;
    my $class = $ENV{MOD_PERL} ? 'Apache::Cookie' : 'CGI::Cookie';
    eval "use $class;";
    $class->fetch;
}

sub show_error {
    my $app = shift;
    my($error) = @_;
    my $tmpl;
    $error = encode_html($error);
    $error =~ s!(http://\S+)!<a href="$1" target="_blank">$1</a>!g;
    $tmpl = $app->load_tmpl('error.tmpl') or
        return "Can't load error template; got error '" . $app->errstr .
               "'. Giving up. Original error was <pre>$error</pre>";
    $tmpl->param(ERROR => $error);
    $tmpl->output;
}

sub run {
    my $app = shift;
    my($return_content) = @_;
    my $q = $app->{query};
    my($body);
    eval {
        my $err;
        eval { $err = $q->cgi_error };
        unless ($@) {
            if ($err && $err =~ /^413/) {
                die "You uploaded a file larger than 1MB. " .
                    "This is not allowed.\n";
            }
        }

    REQUEST:
    {
        if ($app->{requires_login}) {
        LOGIN:
        {
            my($author, $first_time) = $app->login;
            if ($author) {
                $app->{author} = $author;
                if ($first_time) {
                    my $remember = $q->param('remember') ? 1 : 0;
                    my %arg = (
                        -name => 'user',
                        -value => join('::', $author->name, $author->password,
                                             $remember),
                    );
                    $arg{-expires} = '+10y' if $remember;
                    $app->bake_cookie(%arg);
                }
                last LOGIN if $app->is_authorized;
            }
            my $tmpl = $app->load_tmpl('login.tmpl') or
                $body = $app->show_error( $app->errstr ), last REQUEST;
            $tmpl->param(ERROR => $app->errstr);
            ## For bookmarklets, we need to pass-thru text, title, and
            ## href vars
            if ($q->param('is_bm')) {
                $tmpl->param(text => $q->param('text'));
                $tmpl->param(link_title => $q->param('link_title'));
                $tmpl->param(link_href => $q->param('link_href'));
                $tmpl->param(bm_show => $q->param('bm_show'));
            }
            $body = $tmpl->output;
            last REQUEST;
        }  ## end LOGIN block
        }

        my $mode = $q->param('__mode') || $app->{default_mode};
        my $code = $app->{vtbl}{$mode} or
            $app->error("Unknown action $mode");
        if ($code) {
            $body = $code->($app);
        }
        unless (defined $body || $app->{redirect}) {
            $body = $app->show_error( $app->errstr );
        }
    }  ## end REQUEST block
    };
    if ($@) {
        $body = $app->show_error($@);
    }

    ## Add the Pragma: no-cache header.
    ## WEIRD: for CGI::cache, any true argument to cache means NO cache
    $q->cache(1);

    if (my $url = $app->{redirect}) {
        print $q->redirect(-uri => $url, %{ $app->{headers} });
    } else {
        $body = "<pre>$app->{trace}</pre>" . $body if $app->{trace};
        unless ($return_content) {
            print $q->header(%{ $app->{headers} });
            print $body;
        }
    }
    $return_content ? $body : 1;
}

sub load_tmpl {
    my $app = shift;
    my($file, @p) = @_;
    my $path = $app->{cfg}->TemplatePath;
    my $tmpl;
    eval {
        $tmpl = HTML::Template->new_file(
            File::Spec->catfile($path, $app->{template_dir}, $file),
            path => [ File::Spec->catdir($path, $app->{template_dir}) ],
            die_on_bad_params => 0, global_vars => 1, @p);
    };
    return $app->error("Loading template '$file' failed: $@") if $@;
    my $blog_id = $app->{query}->param('blog_id');
    $tmpl->param(SCRIPT_URL => $app->uri);
    $tmpl->param(SCRIPT_PATH => $app->path);
    $tmpl->param(SCRIPT_FULL_URL => $app->base . $app->uri);
    if ($app->{author}) {
        $tmpl->param(AUTHOR_ID => $app->{author}->id);
        $tmpl->param(AUTHOR_NAME => $app->{author}->name);
    }
    if ($blog_id) {
        my $blog = MT::Blog->load($blog_id);
        $tmpl->param(BLOG_NAME => $blog->name);
        $tmpl->param(BLOG_ID => $blog_id);
        $tmpl->param(BLOG_URL => $blog->site_url);
    }
    if ($app->{query}->param('is_bm')) {
        $tmpl->param(is_bookmarklet => 1);
    }
    my $spath = $app->{cfg}->StaticWebPath || $app->path;
    $tmpl->param(static_uri => $spath);
    my $vers = MT->VERSION;
    if ($vers =~ /^0\.0*/) {
        $vers = "b $vers";
    }
    $tmpl->param(mt_version => $vers);
    $tmpl->param(DATE_NOW_FORMATTED =>
        strftime("%Y.%m.%d", offset_time_list(time, $blog_id)));
    $tmpl;
}

sub build_page {
    my $app = shift;
    my($file, $param) = @_;
    my $tmpl = $app->load_tmpl($file) or return;
    for my $key (keys %$param) {
        $tmpl->param($key, $param->{$key});
    }
    $tmpl->output;
}

## Path/server/script-name determination methods

sub base {
    my $app = shift;
    return $app->{__host} if exists $app->{__host};
    if ($CGI::VERSION >= 2.73) {
        return $app->{__host} = $app->{query}->url(-base => 1);
    }
    my $path = $app->{cfg}->CGIPath;
    if ($path =~ m!^(https?://[^/]+)!i) {
        (my $host = $1) =~ s!/$!!;
        return $app->{__host} = $host;
    }
    '';
}

sub path {
    my $app = shift;
    return $app->{__path} if exists $app->{__path};
    my $path = $app->{cfg}->CGIPath;
    if ($path =~ m!^https?://[^/]+(/.*)$!i) {
        $path = $1;
    }
    $path .= '/' unless substr($path, -1, 1) eq '/';
    $app->{__path} = $path;
}

sub script {
    my $app = shift;
    return $app->{__script} if exists $app->{__script};
    return (split /\//, $ENV{SCRIPT_NAME})[-1];
}

sub uri { $_[0]->path . $_[0]->script }

sub redirect {
    my $app = shift;
    my($url) = @_;
    unless ($url =~ m!^https?://!i) {
        $url = $app->base . $url;
    }
    $app->{redirect} = $url;
    return;
}

sub log {
    my $app = shift;
    my($msg) = @_;
    my $log = MT::Log->new;
    $log->message($msg);
    $log->ip($ENV{REMOTE_ADDR});
    $log->save;
}

sub trace { $_[0]->{trace} .= "@_" }

1;
